import requests
from pyquery import PyQuery
import csv

def get_topics(topic, maxpage, cookies, csvf, startpage=1, encoding='utf-8'):
    """
    获取微博某topic的所有相关微博内容信息
    :param topic: 微博话题。比如#华为#，那么topic='华为'
    :param maxpage: 某微博话题最大页码数
    :param cookies: 自己微博的cookies，需要自己在浏览器先登录微博。之后F12开发者工具打开开发者工具network面板查找对应的cookies
    :param csvf:  csv文件路径；保存数据到csv中
    :param startpage: 抓微博话题时默认从第一页开始抓。一般不设置该参数，除非爬虫爬了一部分遇到问题后， 再次启动爬虫时设置startpage参数，避免重复采集
    :param encoding: csv文件编码方式，默认用utf-8 (推荐); 也可以更改为gbk
    :return:
    """
    csvf = open(csvf, 'a+', encoding=encoding, newline='')
    fieldnames = ['昵称', '用户主页', '微博id', '微博',
                  '评论数', '点赞数', '转发数','收藏数']
    writer = csv.DictWriter(csvf, fieldnames=fieldnames)

    for page in range(startpage, maxpage+1):
        url = 'https://s.weibo.com/weibo?q={topic}&page={page}'.format(topic=topic, page=page)
        resp = requests.get(url, cookies=cookies)
        doc = PyQuery(resp.text)
        for feed in doc.items('.card-wrap'):
            try:
                info = feed('.info div').eq(1)('a')
                nickname = info.attr('nick-name')
                userhomelink = 'https://weibo.com/' + info.attr('href')
                mid = feed.attr('mid')  # 某微博对应的id
                content = feed('.content .txt').text()
                shoucang = feed('.card-act li').eq(0).text().replace('收藏', '').replace(' ', '')
                try:
                    shoucang = int(shoucang)
                except:
                    shoucang = 0
                zhuanfa = feed('.card-act li').eq(1).text().replace('转发', '').replace(' ', '')
                try:
                    zhuanfa = int(zhuanfa)
                except:
                    zhuanfa = 0

                pinglun = feed('.card-act li').eq(2).text().replace('评论', '').replace(' ', '')
                try:
                    pinglun = int(pinglun)
                except:
                    pinglun = 0

                dianzan = feed('.card-act li').eq(3).text()
                try:
                    dianzan = int(dianzan)
                except:
                    dianzan = 0

            except:
                pass

            data = {'昵称': nickname,
                    '用户主页': userhomelink,
                    '微博id': mid,
                    '微博': content,
                    '评论数': pinglun,
                    '转发数': zhuanfa,
                    '点赞数': dianzan,
                    '收藏数': shoucang}

            writer.writerow(data)
            print('第'+str(page)+'页', data)
            print()

    csvf.close()